@@ -14,14 +14,14 @@ __KTOOL_BEGIN_NAMESPACE
 #define ITEM_SEP        1
 #define ITEM_LINE       2
 
-struct verticalmenuitem {
-    string text;
-    int color, kind;
-    void *ref;
-};
-
 class verticalmenu: public abstractuicontrol {
     private:
+	struct verticalmenuitem {
+	    string text;
+	    int color, kind;
+	    void *ref;
+	};
+
 	vector<verticalmenuitem> items;
 	textwindow window;
 
@@ -34,17 +34,19 @@ class verticalmenu: public abstractuicontrol {
 	void showall();
 
 	void checkclear();
-        void intredraw();
+	void intredraw();
 
     public:
-	bool clearonfocuslost, cycled;
+	bool clearonfocuslost;
+	bool cycled;
+	bool exitonedges;
 
-	int (*otherkeys)(verticalmenu *caller, int k);
+	int (*otherkeys)(verticalmenu &caller, int k);
 	    // function called on pressing of non-standard menu keys
 	    // should return a number of selected menu item or -1 to
 	    // continue browsing
 
-	void (*idle)(verticalmenu *caller);
+	void (*idle)(verticalmenu &caller);
 
 	verticalmenu(int px1, int py1, int px2, int py2, int pncolor, int pscolor);
 	verticalmenu(int pncolor = 0, int pscolor = 0);
@@ -63,10 +65,13 @@ class verticalmenu: public abstractuicontrol {
 	//   \009 == URCORNER [ ^| ]
 
 	void additemf(const char *fmt, ...);
+	void additem(const string text);
+
+	void additem(int color, void *ref, const string text);
 	void additemf(int color, void *ref, const char *fmt, ...);
 
-	void additem(string text);
-	void additem(int color, void *ref, string text);
+	void additem(int color, int ref, const string text);
+	void additemf(int color, int ref, const char *fmt, ...);
 
 	void addline();
 	void addline(int color, const char *fmt = 0, ...);
@@ -83,7 +88,7 @@ class verticalmenu: public abstractuicontrol {
 	int getpos();
 	void getpos(int &cur, int &first);
 	void setpos(int cur, int first = -1);
-	void setcolor(int pos, int color);
+	void setitemcolor(int pos, int color);
 	void setwindow(textwindow nwindow);
 	void remove(int pos);
 
@@ -91,12 +96,12 @@ class verticalmenu: public abstractuicontrol {
 	void scale();
 	void close();
 	void setcoord(int nx1, int ny1, int nx2, int ny2);
-	void setcolors(int pncolor, int pscolor);
+	void setcolor(int pncolor, int pscolor);
 
     friend class horizontalmenu;
 };
 
-class horizontalmenuitem { public:
+struct horizontalmenuitem {
     string text;
     int color;
     verticalmenu menu;
@@ -112,7 +117,7 @@ class horizontalmenu: public abstractuicontrol {
 	int ncolor, scolor, fcolor, coordy, coordx, selected;
 	bool finished;
 
-	static int menu_otherkeys(verticalmenu *ref, int k);
+	static int menu_otherkeys(verticalmenu &ref, int k);
 
 	void draw();
 	void moveelem(int old);
@@ -123,8 +128,8 @@ class horizontalmenu: public abstractuicontrol {
 	int menulen(int n);
 
     public:
-	bool (*otherkeys)(horizontalmenu *caller, int k);
-	void (*idle)(horizontalmenu *caller);
+	bool (*otherkeys)(horizontalmenu &caller, int k);
+	void (*idle)(horizontalmenu &caller);
 
 	horizontalmenu(int x, int y, int normcolor, int selcolor, int framecolor);
 	horizontalmenu();
