@@ -5,15 +5,17 @@
 #include "conf.h"
 
 #include <string>
+#include <vector>
+
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <ctype.h>
 #include <signal.h>
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/ioctl.h>
 #include <ncurses.h>
+#include <ctype.h>
 
 #undef box
 #undef clear
@@ -26,6 +28,12 @@
 #define boldcolor(c)    COLOR_PAIR(c)|A_BOLD
 #define color(c)        COLOR_PAIR(c)
 
+#ifdef LOCALES_HACK
+#define KT_DISP_FILTER(c)       ( ((c > ' ') && (c != '\177')) ? c : ' ' )
+#else
+#define KT_DISP_FILTER(c)       (!iscntrl(c) ? c : ' ')
+#endif
+
 #define VLINE           kintf_graph ? ACS_VLINE         : '|'
 #define HLINE           kintf_graph ? ACS_HLINE         : '-'
 #define ULCORNER        kintf_graph ? ACS_ULCORNER      : '+'
@@ -54,21 +62,20 @@
 #define CONTROL_PRESSED 4
 
 extern bool kintf_graph, kintf_refresh;
+extern void (*kt_resize_event)(void);
 
 void printchar(char c);
-void printstring(string s);
+void printstring(const string s);
 int string2key(const string adef);
 
 const string makebidi(const string buf, int lpad = 0);
+int findcolor(const string s);
 
 __KTOOL_BEGIN_C
 
 void kinterface();
 void kendinterface();
-  
-extern void (*kinputidle)(void);
-char *kinput(unsigned int size, char *buf);
-  
+
 int keypressed();
 int emacsbind(int k);
 
