@@ -1,7 +1,7 @@
 /*
 *
 * kkconsui common routines
-* $Id: conscommon.cc,v 1.4 2001/08/06 21:32:45 konst Exp $
+* $Id: conscommon.cc,v 1.11 2001/11/21 14:45:39 konst Exp $
 *
 * Copyright (C) 1999-2001 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -26,7 +26,7 @@
 
 bool kintf_graph = true, kintf_refresh = true;
 
-void (*kinputidle)(void);
+void (*kt_resize_event)(void);
 
 void kreinit(int sn) {
     struct winsize size;
@@ -34,6 +34,9 @@ void kreinit(int sn) {
     if(ioctl(fileno(stdout), TIOCGWINSZ, &size) == 0) {
 	resizeterm(size.ws_row, size.ws_col);
 	wrefresh(curscr);
+
+	if(kt_resize_event)
+	    kt_resize_event();
     }
 
     signal(sn, &kreinit);
@@ -52,6 +55,7 @@ void kinterface() {
 //      nodelay(stdscr, TRUE);
     ESCDELAY = 0;
     signal(SIGWINCH, &kreinit);
+    kt_resize_event = 0;
 }
 
 void kendinterface() {
@@ -128,87 +132,17 @@ int emacsbind(int k) {
 }
 
 void printchar(char c) {
-    printw("%c", !iscntrl(c) ? c : ' ');
+    printw("%c", KT_DISP_FILTER(c));
 }
 
-void printstring(string s) {
+void printstring(const string s) {
     int i;
+    string ds;
 
     for(i = 0; i < s.size(); i++)
-    if(iscntrl(s[i])) s[i] = ' ';
+	ds += KT_DISP_FILTER(s[i]);
 
-    printw("%s", s.c_str());
-}
-
-char *kinput(unsigned int size, char *buf) {
-    int c, x, y;
-    unsigned int offset = 0, i;
-    char t[size], go, finish = 0;
-
-    refresh();
-    strcpy(buf, "");
-
-    while(!finish) {
-	if(kinputidle) go = keypressed(); else go = 1;
-      
-	if(go) switch(c = getkey()) {
-	    case '\r' :
-		finish = 1;
-		break;
-	  
-	    case KEY_LEFT :
-		if(offset > 0) {
-		    offset--;
-		    kgotoxy(kwherex() - 1, kwherey());
-		}
-		break;
-
-	    case KEY_RIGHT :
-		if(offset < strlen(buf)) printchar(buf[offset++]);
-		break;
-
-	    case KEY_DC :
-		if(strlen(buf) > 0 && offset < strlen(buf)) {
-		    x = kwherex(); y = kwherey();
-		    for(i = offset + 1; i < strlen(buf); i++) printchar(buf[i]);
-		    printw(" ");
-		    kgotoxy(x, y);
-		    refresh();
-
-		    t[0] = 0;
-		    for(i = 0; i < strlen(buf); i++)
-		    if(i != offset) sprintf(t, "%s%c", t, buf[i]);
-		    strcpy(buf, t);
-		}
-		break;
-	  
-	    case KEY_BACKSPACE : case 8 :
-		if(strlen(buf) && offset > 0) {
-		    offset--;
-		    kgotoxy(kwherex() - 1, kwherey());
-		    x = kwherex(); y = kwherey();
-		    for(i = offset + 1; i < strlen(buf); i++) printchar(buf[i]);
-		    printw(" ");
-		    kgotoxy(x, y);
-		    refresh();
-
-		    t[0] = 0;
-		    for(i = 0; i < strlen(buf); i++)
-		    if(i != offset) sprintf(t, "%s%c", t, buf[i]);
-		    strcpy(buf, t);
-		}
-		break;
-      
-	    default :
-		if(strlen(buf) < size && c > 31 && c < 255) {
-		    printchar(c);
-		    refresh();
-		    offset++;
-		    sprintf(buf, "%s%c", buf, c);
-		}
-	} else if(kinputidle) (*kinputidle)();
-    }
-    return buf;
+    printw("%s", ds.c_str());
 }
 
 int kwherex() {
@@ -282,7 +216,7 @@ int string2key(const string adef) {
 
     r = 0;
     d = adef;
-    if((ck = d).find("-") != -1) ck = getrword(d);
+    if((ck = d).find("-") != -1) ck = getrword(d, "-");
 
     if(!ck.empty()) {
 	if((ck.size() > 1) && (toupper(ck[0]) == 'F')) {
@@ -348,3 +282,15 @@ const string makebidi(const string buf, int lpad = 0) {
 }
 
 #endif
+
+int findcolor(const string s) {
+    int i;
+    string::iterator is;
+    string colors[] = {"BLACK", "RED", "GREEN", "YELLOW", "BLUE", "MAGENTA", "CYAN", "WHITE", ""};
+    string ls = s;
+
+    for(is = ls.begin(); is != ls.end(); *is = toupper(*is), is++);
+    for(i = 0; !colors[i].empty() && (ls != colors[i]); i++);
+
+    return i;
+}
