@@ -49,6 +49,8 @@
 #define CURCOL        (curfile ? (curfile->sx+curfile->x) : 0)
 #define CURSTRING     (char *) curfile->lines->at(CURLINE)
 #define CSTRLEN       strlen(CURSTRING ? CURSTRING : "")
+#define UPDATECURRENTLINE       { kgotoxy(x1, y1+curfile->y); showline(CURLINE, curfile->sx, x2-x1); }
+
 #define MAX_STRLEN    10240
 #define ALONE_DELIM   " ;(){}[].,:-+*/^?!=<>"
 #define NONCHAR_DELIM " ;(){}[].,:-+*/^?!=<>\"'"
@@ -58,12 +60,12 @@
 #define EM_CTRL       4
 #define EM_MANUAL     8
 
-#define CHECKLOADED	if(!getfcount()) return;
+#define CHECKLOADED     if(!getfcount()) return;
 
 texteditor::texteditor():
 otherkeys(0), fn(-1), wrap(false), abscol(0), idle(0),
 insertmode(true), undolog(true), show(true), curfile(0),
-prevshift(false) {
+prevshift(false), smarttab(true) {
 
     files = new linkedlist;
     files->freeitem = &editfilefree;
@@ -73,21 +75,29 @@ texteditor::~texteditor() {
     delete files;
 }
 
-int texteditor::load(string buf, const string id) {
+int texteditor::load(const string abuf, const string id) {
     int newfn = addwindow(strdup(id.c_str())), k;
+    string buf = abuf;
+    string::iterator is;
     vector<string> lst;
     vector<string>::iterator i;
 
     setfnum(newfn);
-    while((k = buf.find("\r")) != -1) buf.replace(k, 1, "");
+
+    is = buf.begin();
+    while((is = ::find(buf.begin(), buf.end(), '\r')) != buf.end()) {
+	buf.erase(is);
+	is++;
+    }
+
     breakintolines(buf, lst, wrap ? x2-x1 : 0);
 
     curfile->lines->empty();
     for(i = lst.begin(); i != lst.end(); i++) {
 	curfile->lines->add(strdup(i->c_str()));
     }
-
     curfile->lines->add(strdup(""));
+
     return newfn;
 }
 
@@ -256,7 +266,7 @@ int texteditor::addwindow(char *id) {
     return files->count-1;
 }
 
-void texteditor::modification(tundoaction action, string data,
+void texteditor::modification(tundoaction action, const string data,
 bool connected = false, int curx = -1, int cury = -1) {
     if(undolog && !data.empty()) {
 	undorecord *ur = new undorecord;
@@ -307,7 +317,7 @@ void texteditor::addhighlight(int nscheme, string text, int color, hl_kind kind)
 	    s.synt_quote = text;
 	    s.qcolor = color;
 
-	    if(i = text.find(" ")) {
+	    if((i = text.find(" ")) != -1) {
 		s.synt_qescape = text.substr(i+1);
 		s.synt_quote.resize(i);
 	    }
@@ -527,14 +537,28 @@ void texteditor::insert(FILE *f) {
     }
 }
 
-void texteditor::insert(char *p) {
+void texteditor::insert(const string abuf) {
     CHECKLOADED;
-    if(p) {
-	char *sl = strdup(CURSTRING), *el = strdup(CURSTRING+CURCOL);
-	char *curpos = p, buf[1024], *s;
-	int line = 0;
+
+    string sbuf;
+    vector<string> lst;
+    vector<string>::iterator is;
+
+    sbuf = abuf;
+
+    if(sbuf.find_first_of("\n\t") != -1) {
+	breakintolines(sbuf, lst, 0);
+	for(sbuf = "", is = lst.begin(); is != lst.end(); is++) {
+	    sbuf += *is + "\n";
+	}
+    }
+
+    if(!sbuf.empty()) {
+	char *sl = strdup(CURSTRING), *el = strdup(CURSTRING+CURCOL), buf[1024], *s;
+	const char *curpos = sbuf.c_str();
 	bool firstpass = true;
-    
+	int line = 0;
+
 	sl[CURCOL] = 0;
 
 	while(1) {
@@ -562,7 +586,7 @@ void texteditor::insert(char *p) {
 	delete el;
 	delete sl;
 
-	modification(uinsblock, p);
+	modification(uinsblock, sbuf);
     }
 }
 
@@ -633,9 +657,9 @@ void texteditor::scancomments(bool visible) {
 	    while(1) {
 		lsub = strqstr(sub, lc.c_str(), colors.synt_quote.c_str());
 
-                if(tb) rsub = strstr(sub, rc.c_str());
-                else rsub = strqstr(sub, rc.c_str(), colors.synt_quote.c_str());
-		
+		if(tb) rsub = strstr(sub, rc.c_str());
+		else rsub = strqstr(sub, rc.c_str(), colors.synt_quote.c_str());
+
 		if(rsub && (((rsub < lsub) && lsub) || !lsub)) {
 		    sub = rsub;
 		    if(!tb) {
@@ -682,27 +706,46 @@ void texteditor::scancomments(bool visible) {
     }
 }
 
-void texteditor::hl_comment(char *cp, char *txt, int color) {
+int texteditor::hl_comment(char *cp, char *txt, int color) {
+    int r;
     const char *p;
-    if(p = strqstr(cp, txt, "\"'")) hl_comment(cp, p-cp, strlen(cp), color);
+
+    r = 0;
+    if(p = strqstr(cp, txt, "\"'"))
+	r = hl_comment(cp, p-cp, strlen(cp), color);
+
+    return r;
 }
 
-void texteditor::hl_comment(char *cp, int st, int pend, int color) {
-    int i, delcount = 0;
+int texteditor::hl_comment(char *cp, int st, int pend, int color) {
+    int i, delcount, r;
     char ins[5] = "\001 ";
     int origclr = -1;
 
-    if(color) {
-	for(i = 0; (i <= pend) && (i < strlen(cp)); i++)
-	if(cp[i] == 1) origclr = cp[i+++1]; else
-	if(cp[i] == 2) origclr = -1;
+    delcount = r = 0;
 
-	if(pend > strlen(cp)) pend = strlen(cp);
-	if(cp[pend] != 2) strinsert(cp, pend, "\002");
+    if(color && (st <= strlen(cp)) && (pend-st > 0)) {
+/// !!!         for(i = 0; (i <= pend) && (i < strlen(cp)); i++)
+
+	for(i = 0; (i <= pend) && (i < strlen(cp)); i++) {
+	    switch(cp[i]) {
+		case 1: origclr = cp[i+++1]; break;
+		case 2: origclr = -1; break;
+	    }
+	}
+
+	if(pend > strlen(cp))
+	    pend = strlen(cp);
+
+	if(cp[pend] != 2) {
+	    strinsert(cp, pend, "\002");
+	    r++;
+	}
 
 	if(origclr != -1) {
 	    ins[1] = origclr;
 	    strinsert(cp, pend+1, ins);
+	    r += 2;
 	}
 
 	for(i = st; (i < pend) && (i < strlen(cp)); i++) {
@@ -722,7 +765,10 @@ void texteditor::hl_comment(char *cp, int st, int pend, int color) {
 
 	ins[1] = color;
 	strinsert(cp, st, ins);
+	r += 2;
     }
+
+    return r-delcount;
 }
 
 int texteditor::count_clrcodes(char *cp, int pos) {
@@ -742,17 +788,24 @@ int texteditor::count_clrcodes(char *cp, int pos) {
 void texteditor::showline(int ln, int startx, int distance, int extrax = 0) {
     if(!show) return;
 
-    int i, n, inscount, bcolor, sxinscount, printed, j, lastoccur, q, eolstart = 0, npos;
-    char *cs, *cp, *sr, *nr, *r, ins[3] = "\001 ", *buf;
+    int i, n, inscount, bcolor, sxinscount, printed, j, lastoccur, q, eolstart, npos, offs;
+    char *cs, *sr, *nr, *r, ins[3] = "\001 ";
+
+    vector<int> layout;
+    vector<int>::iterator iq;
     vector<hlight>::iterator hi;
+
     const char *p;
 
     if(!(cs = (char *) curfile->lines->at(ln))) return;
-    cp = new char[eolstart = strlen(cs)*3+16];
-    buf = new char[strlen(cs)*3+16];
+    char cp[i = (strlen(cs)+1)*4];
+    char buf[i];
+
+    eolstart = i;
 
     strcpy(cp, cs);
-    inscount = sxinscount = bcolor = buf[0] = 0;
+    buf[0] = 0;
+    inscount = sxinscount = bcolor = 0;
 
     highline *hline = (highline *) curfile->highlines->find(&(i = ln+1), &findhighline);
     
@@ -761,6 +814,7 @@ void texteditor::showline(int ln, int startx, int distance, int extrax = 0) {
 	strinsert(cp, 0, ins);
 	strcat(cp, "\002");
     } else {
+	if(strlen(cp))
 	for(hi = colors.hl.begin(); hi != colors.hl.end(); hi++) {
 	    ins[1] = hi->color;
 	    p = cp;
@@ -775,9 +829,9 @@ void texteditor::showline(int ln, int startx, int distance, int extrax = 0) {
 			}
 
 			if(!strlen(r)) {
-                            r = nr;
-                            continue;
-                        }
+			    r = nr;
+			    continue;
+			}
 
 			p = cp;
 			lastoccur = 0;
@@ -805,14 +859,11 @@ void texteditor::showline(int ln, int startx, int distance, int extrax = 0) {
 		    break;
 
 		case h_symbol:
-		    while(p = strqpbrk(cp, p-cp+lastoccur, hi->text.c_str(),
-		    colors.synt_quote.c_str(), colors.synt_qescape.c_str())) {
-			if(eolstart) eolstart += lastoccur;
-			if(p-cp > eolstart) break;
-			lastoccur = strspn(p, hi->text.c_str());
-			strinsert(cp, p-cp+lastoccur, "\002");
-			strinsert(cp, p-cp, ins);
-			lastoccur += 3;
+		    layout = getsymbolpositions(string(cp).substr(0, eolstart),
+			hi->text, colors.synt_quote, colors.synt_qescape);
+
+		    for(offs = 0, iq = layout.begin(); iq != layout.end(); iq++) {
+			offs += hl_comment(cp, *iq+offs, *iq+offs+1, hi->color);
 		    }
 		    break;
 
@@ -841,42 +892,25 @@ void texteditor::showline(int ln, int startx, int distance, int extrax = 0) {
 	}
 
 	// Quotes highlight ...
-	
+
 	if(!colors.synt_quote.empty()) {
-	    char qchar = 0;
-	    int qpos;
-	    
-	    p = cp;
-	    ins[1] = colors.qcolor;
+	    bool qst;
 
-	    for(qchar = 0; p = strpbrk(p, colors.synt_quote.c_str()); ) {
+	    layout = getquotelayout(string(cp).substr(0, eolstart),
+		colors.synt_quote, colors.synt_qescape);
 
-		// check for "\" before the quote char
-		if(p > cp+1)
-		if(colors.synt_qescape.find(*(p-1)) != -1)
-		if(colors.synt_qescape.find(*(p-2)) == -1) {
-		    p++;
-		    continue;
+	    for(qst = false, offs = 0, iq = layout.begin(); iq != layout.end(); iq++) {
+		qst = !qst;
+
+		if(!qst) {
+		    offs += hl_comment(cp, *(iq-1)+offs,
+			*iq+offs+1, colors.qcolor);
 		}
-
-		if(p-cp > eolstart) {
-		    if(qchar) strinsert(cp, eolstart, "\002");
-		    break;
-		}
-		
-		if(!qchar) {
-		    qchar = *p;
-		    qpos = p-cp;
-		} else if(*p == qchar) {
-		    qchar = 0;
-		    hl_comment(cp, qpos, p-cp+1, colors.qcolor);
-		    p += 3;
-		}
-
-		p++;
 	    }
 
-	    if(qchar) hl_comment(cp, qpos, strlen(cp), colors.qcolor);
+	    if(qst) {
+		hl_comment(cp, *(layout.end()-1)+offs, strlen(cp), colors.qcolor);
+	    }
 	}
 
 	// Blocks ...
@@ -906,12 +940,10 @@ void texteditor::showline(int ln, int startx, int distance, int extrax = 0) {
 		} else n = j = 0;
 
 		if(!(tb->x1 == tb->x2 && tb->y1 == tb->y2)) {
-		    //if(!cp[0]) extrax++;
 		    hl_comment(cp, n, q, tb->color);
 		}
 	    }
 	}
-
     }
 
     // let's count the amount of color codes inserted
@@ -945,9 +977,6 @@ void texteditor::showline(int ln, int startx, int distance, int extrax = 0) {
     if(printed < 0) printed = 0; else
     if(printed > distance) printed = distance;
     mvhline(y1+ln-curfile->sy, x1+extrax+printed, ' ', distance-printed);
-
-    delete cp;
-    delete buf;
 }
 
 void texteditor::draw(int fromline) {
@@ -965,6 +994,7 @@ void texteditor::draw(int fromline) {
 		for(; k < y2-y1+curfile->sy; k++) mvhline(k-curfile->sy+y1, x1, ' ', x2-x1);
 	    }
 	}
+
 	refresh();
     }
 }
@@ -1034,10 +1064,11 @@ void texteditor::eddel(bool usetabs = true) {
 	    }
 
 	    modification(udelchar, deltext);
-
 	    strcut(p, CURCOL, todelete);
+
+	    UPDATECURRENTLINE;
 	    updatecursor();
-	    showline(CURLINE, CURCOL, x2-x1-curfile->x, curfile->x);
+//          showline(CURLINE, CURCOL, x2-x1-curfile->x, curfile->x);
 	    
 	} else {
 	    char *next = (char *) curfile->lines->at(CURLINE+1);
@@ -1319,17 +1350,22 @@ void texteditor::edmove(int k, int options = EM_TAB) {
 		break;
 
 	    case KEY_RIGHT:
-		if(ctrlpressed && (CURLINE != curfile->lines->count-1)) {
+		if(ctrlpressed) {
 		    char *p = CURSTRING+CURCOL+1, *r = 0;
 
 		    if(*(CURSTRING+CURCOL)) {
 			if(r = strpbrk(p, NONCHAR_DELIM))
 			for(; *r && strchr(NONCHAR_DELIM, *r); r++);
 		    }
-		    
-		    if(r) curfile->x += r-p+1; else {
-			if(endofline()) setpos(0, CURLINE+1);
-			else curfile->x = CSTRLEN;
+
+		    if(r) {
+			curfile->x += r-p+1;
+		    } else {
+			if(endofline() && (CURLINE < curfile->lines->count-1)) {
+			    setpos(0, CURLINE+1);
+			} else {
+			    curfile->x = CSTRLEN;
+			}
 		    }
 		} else {
 		    if(CSTRLEN > CURCOL) {
@@ -1486,19 +1522,13 @@ void texteditor::inschar(int k) {
 	    draw();
 
 	} else {
-
 	    curfile->lines->replace(CURLINE, n);
 	    modification(uinschar, np);
-	
-	    int startx;
-	    for(startx = curfile->x; startx && !strchr(WORD_DELIM, n[startx]); startx--);
-	    kgotoxy(x1+startx, y1+curfile->y);
 
-	    showline(CURLINE, curfile->sx+startx, x2-x1-startx, startx);
+	    UPDATECURRENTLINE;
 	    edmove(KEY_RIGHT, 0);
-	    
 	}
-	
+
 	updatecursor();
 	abscol = CURCOL;
     }
@@ -1507,6 +1537,8 @@ void texteditor::inschar(int k) {
 void texteditor::setpos(int col, int line) {
     bool drawneeded = false;
 
+    CHECKLOADED;
+
     if(line >= curfile->lines->count)
 	line = curfile->lines->count-1;
 
@@ -1593,7 +1625,9 @@ int texteditor::open() {
 		if(prevshift) prevshift = curfile->markmode = false;
 	    switch(k) {
 		case '\r':
-		    if(insertmode || (curfile->lines->count == CURLINE+1)) edenter(); else {
+		    if(insertmode || (curfile->lines->count == CURLINE+1)) {
+			edenter(smarttab);
+		    } else {
 			edmove(KEY_DOWN);
 			edmove(KEY_HOME);
 		    }
@@ -1613,7 +1647,7 @@ int texteditor::open() {
 		    eddelword();
 		    break;
 		default:
-		    if(k >= 32 && k < 255) {
+		    if(k >= 32 && k <= 255) {
 			if(!insertmode && !endofline()) eddel();
 			inschar(k);
 		    } else if(otherkeys) {
@@ -1623,7 +1657,7 @@ int texteditor::open() {
 			    case KEY_DC: eddel(); break;
 			} else {
 			    l = fn;
-			    if((*otherkeys)(this, k) == -1) return -1;
+			    if((*otherkeys)(*this, k) == -1) return -1;
 			    if(l != fn) {
 				draw();
 				updatecursor();
@@ -1632,7 +1666,7 @@ int texteditor::open() {
 		    }
 	    }}
 	} else {
-	    if(idle) (*idle)(this);
+	    if(idle) (*idle)(*this);
 	}
     }
 
@@ -1698,7 +1732,7 @@ bool texteditor::ismark() {
 }
 
 char *texteditor::getline(int ln) {
-    return (char *) curfile->lines->at(ln);
+    return (char *) (getfcount() ? curfile->lines->at(ln) : 0);
 }
 
 void texteditor::putline(int ln, const char *newline) {
@@ -1715,6 +1749,7 @@ void texteditor::putline(int ln, const char *newline) {
 }
 
 void texteditor::redraw() {
+    active = (bool) curfile;
     draw();
 }
 
@@ -1921,14 +1956,20 @@ void texteditor::shiftmarkedblock(int delta) {
 
 // --------------------------------------------------------------------------
 
-bool ktool::hlight::operator == (const hl_kind &k) const {
+#ifdef __KTOOL_USE_NAMESPACES
+
+using ktool::hlight;
+
+#endif
+
+bool hlight::operator == (const hl_kind &k) const {
     return kind == k;
 }
 
-bool ktool::hlight::operator != (const hl_kind &k) const {
+bool hlight::operator != (const hl_kind &k) const {
     return kind != k;
 }
 
-bool ktool::hlight::operator < (const hlight &ah) const {
+bool hlight::operator < (const hlight &ah) const {
     return kind == h_eol;
 }
