@@ -1,7 +1,7 @@
 /*
 *
 * kkconsui treeview class
-* $Id: treeview.cc,v 1.4 2001/08/03 09:21:14 konst Exp $
+* $Id: treeview.cc,v 1.6 2001/11/15 16:49:13 konst Exp $
 *
 * Copyright (C) 1999-2001 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -58,28 +58,46 @@ void treeview::init() {
 }
 
 int treeview::addnodef(int parent, int color, void *ref, const char *fmt, ...) {
-    char buf[1024];
-    va_list ap;
-
-    va_start(ap, fmt);
-    vsprintf(buf, fmt, ap);
-    va_end(ap);
-
+    string buf;
+    VGETSTRING(buf, fmt);
     return addnode(parent, color, ref, buf);
 }
 
+int treeview::addnodef(const char *fmt, ...) {
+    string buf;
+    VGETSTRING(buf, fmt);
+    return addnode(buf);
+}
+
 int treeview::addleaff(int parent, int color, void *ref, const char *fmt, ...) {
-    char buf[1024];
-    va_list ap;
-
-    va_start(ap, fmt);
-    vsprintf(buf, fmt, ap);
-    va_end(ap);
-
+    string buf;
+    VGETSTRING(buf, fmt);
     return addleaf(parent, color, ref, buf);
 }
 
-int treeview::addnode(int parent, int color, void *ref, string text) {
+int treeview::addleaff(const char *fmt, ...) {
+    string buf;
+    VGETSTRING(buf, fmt);
+    return addleaf(buf);
+}
+
+int treeview::addnodef(int parent, int color, int ref, const char *fmt, ...) {
+    string buf;
+    VGETSTRING(buf, fmt);
+    return addnode(parent, color, (void *) ref, buf);
+}
+
+int treeview::addleaff(int parent, int color, int ref, const char *fmt, ...) {
+    string buf;
+    VGETSTRING(buf, fmt);
+    return addleaf(parent, color, (void *) ref, buf);
+}
+
+int treeview::addnode(const string text) {
+    return addnode(0, 0, 0, text);
+}
+
+int treeview::addnode(int parent, int color, void *ref, const string text) {
     treeviewnode node;
 
     node.id = idseq++;
@@ -94,7 +112,11 @@ int treeview::addnode(int parent, int color, void *ref, string text) {
     return node.id;
 }
 
-int treeview::addleaf(int parent, int color, void *ref, string text) {
+int treeview::addleaf(const string text) {
+    addleaf(0, 0, 0, text);
+}
+
+int treeview::addleaf(int parent, int color, void *ref, const string text) {
     treeviewnode node;
 
     node.id = idseq++;
@@ -108,6 +130,14 @@ int treeview::addleaf(int parent, int color, void *ref, string text) {
     return node.id;
 }
 
+int treeview::addnode(int parent, int color, int ref, const string text) {
+    return addnode(parent, color, (void *) ref, text);
+}
+
+int treeview::addleaf(int parent, int color, int ref, const string text) {
+    return addleaf(parent, color, (void *) ref, text);
+}
+
 int treeview::getid(void *ref) {
     int id = -1;
     vector<treeviewnode>::iterator i;
@@ -293,7 +323,7 @@ void treeview::setcur(int id) {
     if(!menu.getcount()) genmenu(0);
 
     if((i = find(refdeps.begin(), refdeps.end(), id)) != refdeps.end()) {
-        menu.setpos(i-refdeps.begin());
+	menu.setpos(i-refdeps.begin());
     }
 }
 
