@@ -1,20 +1,17 @@
 #ifndef __ICQCONTACTS_H__
 #define __ICQCONTACTS_H__
 
-#include "cmenus.h"
-
 /*
 *
 * SORT_CONTACTS
 *
-* O Online
-*   o Online
-*   f Free for chat
-*   i Invisible
-*   d Do not disturb
-*   c Occupied
-*   a Away
-*   n N/A
+* o Online
+* f Free for chat
+* i Invisible
+* d Do not disturb
+* c Occupied
+* a Away
+* n N/A
 *
 * _ Offline
 * ! Not in list
@@ -25,32 +22,39 @@
 
 #define SORT_CONTACTS   "#fidocan_!N"
 
+#define SORTCHAR(c) ( \
+    c->getmsgcount() ? '#' : \
+    !c->inlist() ? '!' : \
+    c->getshortstatus() \
+)
+
+#include "cmenus.h"
+
 #include "linkedlist.h"
 #include "icqcontact.h"
+#include "icqcommon.h"
 
 class icqcontacts: public linkedlist {
     protected:
-	linkedlist visuins;
-
 	static int clistsort(void *p1, void *p2);
 
     public:
 	icqcontacts();
 	~icqcontacts();
 
-	void remove(unsigned int uin, bool nonicq = false);
+	void remove(const imcontact adesc);
 	void load();
 	void save();
-	void send();
-	void nonicq(int id);
 	void order();
+	void rearrange();
+	void checkdefault();
 
-	icqcontact* addnew(unsigned int uin,
-	    bool notinlist = true,
-	    bool nonicq = false);
+	void setoffline(protocolname pname);
 
-	icqcontact *get(unsigned int uin, bool nonicq = false);
-	icqcontact *getseq2(unsigned short seq2);
+	icqcontact* addnew(const imcontact adesc, bool notinlist = true);
+
+	icqcontact *get(const imcontact adesc);
+	icqcontact *getmobile(const string &anumber);
 };
 
 extern icqcontacts clist;
